##########################################
#  IMAGE TINTING FUNCTIONS USING RANGES  #
##########################################

######################################################################
#  Function:  sepiaTint(picture)
#  Description:  Takes in a picture, converts the picture to greyscale, then tints each pixel
#      by increasing the red and increasing the yellow (actually decreases blue).  The RGB
#      values are divided into intensity ranges and different tinting percentages are applied
#      accordingly to provide a better result than uniform tinting.
######################################################################
def sepiaTint(picture):
  #Convert image to greyscale
  greyScaleNew(picture)

  #define range boundaries for color regions
  darktones = range(0, 63)
  midtones = range(64, 191)
  lighttones = range(192, 256)

  #loop through picture to tint pixels
  for p in getPixels(picture):
    r = getRed(p)
    b = getBlue(p)

    #only need to check red value since greyscale sets rgb equal
    #tint colors in darktones
    if(r in darktones):
      r = r*1.1
      b = b*0.9

    #tint colors in midtones
    if(r in midtones):
      r = r*1.15
      b = b*0.85

    #tint colors in lighttones
    if(r in lighttones):
      r = min(r*1.08, 255) #color value could exceed 255, so we take the min
      b = b*0.93

    #update the pixel's new color values
    setRed(p, r)
    setBlue(p, b)

  #refresh the picture to see the results
  repaint(picture)
      
######################################################################
#  Function:  cyanTint(picture)
#  Description:  Takes in a picture, converts the picture to greyscale, then tints each pixel
#      by increasing the blue and increasing the green.  The RGB values are divided into 
#      intensity ranges and different tinting percentages are applied accordingly to provide 
#      a better result than uniform tinting.
######################################################################
def cyanTint(picture):
  #Convert image to greyscale
  greyScaleNew(picture)

  #define range boundaries for color regions
  darktones = range(0, 63)
  midtones = range(64, 191)
  lighttones = range(192, 256)

  #loop through picture to tint pixels
  for p in getPixels(picture):
    g = getGreen(p)
    b = getBlue(p)

    #tint shadows
    if(g in darktones):
      g = g*1.1
      b = b*1.1

    #tint midtones
    if(g in midtones):
      g = g*1.15
      b = b*1.15

    #tint colors in lighttones
    if(g in lighttones):
      g = min(g*1.1, 255) #color value could exceed 255, so we take the min
      b = min(b*1.1, 255) #color value could exceed 255, so we take the min

    #set the new color values
    setBlue(p, b)
    setGreen(p, g)

  #refresh the picture to see the results
  repaint(picture)

######################################################################
#  Function:  magentaTint(picture)
#  Description:  Takes in a picture, converts the picture to greyscale, then tints each pixel
#      by increasing the blue and increasing the red.  The RGB values are divided into 
#      intensity ranges and different tinting percentages are applied accordingly to provide 
#      a better result than uniform tinting.
######################################################################
def magentaTint(picture):
  #Convert image to greyscale
  greyScaleNew(picture)

  #define range boundaries for color regions
  darktones = range(0, 63)
  midtones = range(64, 191)
  lighttones = range(192, 256)

  #loop through picture to tint pixels
  for p in getPixels(picture):
    r = getRed(p)
    b = getBlue(p)

    #tint darktones
    if(r in darktones):
      r = r*1.15
      b = b*1.15

    #tint midtones
    if(r in midtones):
      r = r*1.2
      b = b*1.2

    #tint lighttones
    if(r in lighttones):
      r = min(r*1.1, 255) #color value could exceed 255, so we take the min
      b = min(b*1.1, 255) #color value could exceed 255, so we take the min

    #set the new color values
    setBlue(p, b)
    setRed(p, r)

  #refresh the picture to see the results
  repaint(picture)

######################################################################
#  Function:  greenTint(picture)
#  Description:  Takes in a picture, converts the picture to greyscale, then tints each pixel
#      by increasing the green.  The RGB values are divided into intensity ranges and 
#      different tinting percentages are applied accordingly to provide a better result than 
#      uniform tinting.
######################################################################
def greenTint(picture):
  #Convert image to greyscale
  greyScaleNew(picture)

  #define range boundaries for color regions
  darktones = range(0, 63)
  midtones = range(64, 191)
  lighttones = range(192, 256)

  #loop through picture to tint pixels
  for p in getPixels(picture):
    g = getGreen(p)

    #tint darktones
    if(g in darktones):
      g = g*1.12

    #tint midtones
    if (g in midtones):
      g = g*1.18

    #tint highlights
    if(g in lighttones):
      g = min(g*1.1, 255) #color value could exceed 255, so we take the min

    #set the new color value
    setGreen(p, g)

  #refresh the picture to see the results
  repaint(picture)

######################################################################
#  Function:  quadColorsTint(picture)
#  Description:  Takes in a picture, converts the picture to greyscale, then divides the 
#      picture into 4 quadrants.  Each quadrant is tinted a different color.  The first quadrant 
#      is tinted red, the second green, the third blue, and the fourth yellow.  The RGB values 
#      are divided into intensity ranges and different tinting percentages are applied 
#      accordingly to provide a better result than uniform tinting.
######################################################################
def quadColorsTint(picture):
  #Convert image to greyscale
  greyScaleNew(picture)

  #define range boundaries for color regions
  darktones = range(0, 63)
  midtones = range(64, 191)
  lighttones = range(192, 256)

  #get the height and width
  height = getHeight(picture)
  width = getWidth(picture)

  #divide in half to compute quadrants
  xDelta = width/2
  yDelta = height/2

  #first quad red tint
  for x in range(0, xDelta):
    for y in range(0, yDelta):
      p = getPixel(picture, x, y)
      r = getRed(p)

      #tint darktones
      if (r in darktones):
        r = r*1.15

      #tint midtones
      if (r in midtones):
        r = r*1.2

      #tint lighttones
      if (r in lighttones):
        r = min(r*1.1, 255) #color value could exceed 255, so we take the min

      #set the new color value
      setRed(p, r)

  #second quad green tint
  for x in range(xDelta, width):
    for y in range(0, yDelta):
      p = getPixel(picture, x, y)
      g = getGreen(p)

      #tint darktones
      if(g in darktones):
        g = g*1.15

      #tint midtones
      if(g in midtones):
        g = g*1.2

      #tint lightones
      if(g in lighttones):
        g = min(g*1.1, 255) #color value could exceed 255, so we take the min

      #set the new color value
      setGreen(p, g)
      
  #third quad blue tint
  for x in range(0, xDelta):
    for y in range(yDelta, height):
      p = getPixel(picture, x, y)
      b = getBlue(p)

      #tint darktones
      if (b in darktones):
        b = b*1.15

      #tint midtones
      if (b in midtones):
        b = b*1.2

      #tint lighttones
      if (b in lighttones):
        b = min(b*1.1, 255) #color value could exceed 255, so we take the min

      #set the new color value
      setBlue(p, b)

  #fourth quad yellow tint
  for x in range(xDelta, width):
    for y in range(yDelta, height):
      p = getPixel(picture, x, y)
      b = getBlue(p)

      #tint darktones
      if (b in darktones):
        b = b*0.85

      #tint midtones
      if (b in midtones):
        b = b*0.8

      #tint lighttones
      if (b in lighttones):
        b = b*0.9

      #set the new color value
      setBlue(p, b)

  #refresh the picture to see the results
  repaint(picture)

######################################################################
#  Function: greyScale(picture)
######################################################################
def greyScale(picture):
  for p in getPixels(picture):
    intensity = (getRed(p)+getGreen(p)+getBlue(p))/3
    setColor(p, makeColor(intensity, intensity, intensity))

######################################################################
#  Function: greyScaleNew(picture)
######################################################################
def greyScaleNew(picture):
  for p in getPixels(picture):
    newRed = getRed(p) * 0.299
    newGreen = getGreen(p) * 0.587
    newBlue = getBlue(p) * 0.114
    luminance = newRed + newGreen + newBlue
    setColor(p, makeColor(luminance, luminance, luminance))





