#sets up the images and feature lines
def main():
  sourceImage = makePicture(pickAFile())
  destinationImage = makePicture(pickAFile())

  sourceLines = [[[113, 156], [147, 165]], #source line 1
                  [[173, 169], [201, 168]], #source line 2
                  [[159, 172], [159, 206]], #source line 3
                  [[138, 219], [171, 221]], #source line 4
                  [[165, 3], [183, 145]], #source line 5
                  [[139, 252], [147, 228]]] #source line 6

  destinationLines = [[[95, 155], [130, 157]], #destination line 1
                      [[167, 160], [196, 156]], #destination line 2
                      [[146, 162], [148, 203]], #destination line 3
                      [[108, 221], [175, 223]], #destination line 4
                      [[138, 8], [147, 66]], #destination line 5
                      [[142, 275], [145, 245]]] #destination line 6

  morphMain(sourceImage, destinationImage, sourceLines, destinationLines, 6)

########################
#  MORPHING FUNCTIONS  #
########################

################################################################################
# Function name: morphMain
# Parameters:
#     -sourceImage: the original image before morphing
#     -destinationImage: the final image we are morphing to
#     -sourceLines: the feature lines in the source image
#     -destinationLines: the feature lines in the destination image
#     -numLines: the total number of feature lines
# Description:
#    A main function that sets up everything needed to perform the morph
#    including number of frames, output folder, calculation of line 
#    interpolation, the call to the morph function, and writing the new
#    images out to files.  Also prints status information for each frame 
#    including frame number and feature lines.
################################################################################
def morphMain(sourceImage, destinationImage, sourceLines, destinationLines, numLines, filename="morph"):
  #number of morphing frames between the images
  defaultFrameNumber = 5

  #set folder for output images
  setMediaFolder()

  #write out first and last frames
  writePictureTo(sourceImage, str(filename) + "0.jpg") #first frame is the source image
  writePictureTo(destinationImage, str(filename) + str(defaultFrameNumber+1) + ".jpg") #last frame is the destination image

  #loop through number of frames and call morphing function
  for i in range(0, defaultFrameNumber):
    #calculate interpolated lines
    interpLines = interpolateLines(sourceLines, destinationLines, numLines, i, defaultFrameNumber)

    #print updated processing status
    printNow("Morphing Frame: " + str(i+1))
    printNow("Source Image Feature Lines: " + str(sourceLines))
    printNow("Interpolated Feature Lines: " + str(interpLines))
    printNow("Destination Image Feature Lines: " + str(destinationLines))

    #call the morph function and save result into new image
    morphImage = imageMorph(sourceImage, destinationImage, sourceLines, interpLines, destinationLines, numLines, i, defaultFrameNumber)
    #write the current frame to file
    writePictureTo(morphImage, str(filename) + str(i+1) + ".jpg")

################################################################################
# Function name: imageMorph
# Parameters:
#     -sourceImage: the original image before morphing
#     -destinationImage: the final image we are morphing to
#     -sourceLines: the feature lines in the source image
#     -interpLines: the intermediate feature lines
#     -destinationLines: the feature lines in the destination image
#     -numLines: the total number of feature lines
#     -currFrameNum: the current frame being processed
#     -totalFrameNum: the total number of frames in the morph
# Return:
#    A new image that is warped and cross dissolved according to the 
#    attributes for the current frame
# Description:
#    A morphing function based on the algorithm created by Beier and Neely 
#    as described in the 1992 SIGGRAPH paper "Feature-Based Image 
#    Metamorphosis" (http://www.hammerhead.com/thad/morph.html).
#    The technique uses feature lines in the source image and destination 
#    image to warp the two images towards the lines while cross-dissolving 
#    the images to blend them over time.
################################################################################
def imageMorph(sourceImage, destinationImage, sourceLines, interpLines, destinationLines, numLines, currFrameNum, totalFrameNum):
  #CONSTANTS
  smoothness = 0.5 #smoothness of warping
  relativeLineStrength = 1.25 #relative line strength
  sizeLineStrength = 0.25 #line size strength

  #get width and height of each image
  sourceWidth = sourceImage.getWidth()
  sourceHeight = sourceImage.getHeight()
  destinationWidth = destinationImage.getWidth()
  destinationHeight = destinationImage.getHeight()

  #check if images are the same size
  if(sourceWidth != destinationWidth or sourceHeight != destinationHeight):
    printNow("Cannot morph, both images must be the same size")
    return;

  #create new blank image
  newImage = makeEmptyPicture(sourceHeight, sourceWidth)
  show(newImage)

  #calculate the cross-dissolve intensity for each image
  destinationImageWeight = float(2 * (currFrameNum+1) * (1.0/(totalFrameNum+1)))
  sourceImageWeight = float(2.0 - destinationImageWeight)

  #loop through the pixels of both images
  for x in range(0, sourceWidth):
    for y in range(0, sourceHeight):

      #get pixel values
      sourcePixel = getPixel(sourceImage, x, y) #get current pixel from source image
      destPixel = getPixel(destinationImage, x, y) #get current pixel from destination image
      newPixel = getPixel(newImage, x, y) #get current pixel from new image
      X = [x, y] #get x and y for the current pixel

      #create vectors and weights
      DSUM = [0, 0] #intialize source displacement sum to 0
      DSUM2 = [0, 0] #intialize destination displacement sum to 0
      weightsum = 0 #intialize source weight sum to 0
      weightsum2 = 0 #intialize source weight sum to 0

      #loop through list of lines and process each feature line in the images
      for i in range(0, numLines):
        #set the vector for the current line we are warping towards
        P = interpLines[i][0] #destination line start point
        Q = interpLines[i][1] #destination line end point
        PQ = vectorSubtraction(Q, P) #define destination line segment

        #calculate the u and v to warp the destination image
        u = calcU(X, P, Q) #u is the value along the feature line
        v = calcV(X, P, Q) #v is the perpendicular distance of the current pixel to the line

        #set the vector for the current line in the source image we are warping from
        Pprime = sourceLines[i][0] #source line start point
        Qprime = sourceLines[i][1] #source line start point

        #calculate the new pixel x and y values for the current source image pixel due to warping
        Xprime = calcXPrime(Pprime, Qprime, u, v)

        #set the vector for the current line in destinationImage
        Pprime2 = destinationLines[i][0] #destination line start point
        Qprime2 = destinationLines[i][1] #destination line end point

        #calculate the new pixel x and y values for the current destination image pixel due to warping
        Xprime2 = calcXPrime(Pprime2, Qprime2, u, v)

        #find how far new pixels are from current pixels in the source and destination images
        displacement = vectorSubtraction(Xprime, X) #source image displacement
        displacement2 = vectorSubtraction(Xprime2, X) #destination image displacement

        #check to see if the effect of u
        #if u is greater than 1, distance is the length of the line from the pixel to the end point
        if(u >= 1):
          QX = vectorSubtraction(Q, X)
          dist = magnitude(QX)
        #if u is less than 0, distance is the length of the line from the pixel to the start point
        elif(u <= 0):
          PX = vectorSubtraction(P, X)
          dist = magnitude(PX)
        #otherwise distance is absolute value of v
        else:
          dist = abs(v)

        #find out contribution of current line to pixel warping
        #determine new xy's contribution based on its distance from the line
        length = magnitude(PQ) #length of current line
        weight = pow(length, sizeLineStrength)/(smoothness + dist)
        weight = pow(weight, relativeLineStrength) #weight of current line

        #add the displacement sum for source and destination
        DSUM = vectorAddition(DSUM, scalarMult(displacement, weight))
        DSUM2 = vectorAddition(DSUM2, scalarMult(displacement2, weight))

        #add weight to weightsum
        weightsum = weightsum + weight

        #adjust displacement sum by weightsum
        DSUMweightsum = scalarDivision(DSUM, weightsum)
        DSUMweightsum2 = scalarDivision(DSUM2, weightsum)

        #adjust new source and destination pixels by displacement sums
        Xprime = vectorAddition(X, DSUMweightsum)
        Xprime2 = vectorAddition(X, DSUMweightsum2)

        #get x and y values for new source pixel
        newSourceX = int(Xprime[0])
        newSourceY = int(Xprime[1])

        newDestinationX = int(Xprime2[0])
        newDestinationY = int(Xprime2[1])

        #if new source xy in range of source image, get that pixel
        if(newSourceX in range(0, sourceWidth) and newSourceY in range(0, sourceHeight)):
          sourcePixel = getPixel(sourceImage, newSourceX, newSourceY)

        #if new destination xy in range of destination image, get that pixel
        if(newDestinationX in range(0, sourceWidth) and newDestinationY in range(0, sourceHeight)):
          destPixel = getPixel(destinationImage, newDestinationX, newDestinationY)

        #cross-dissolve source and destination images by the current weights due to current frame number
        r = (sourcePixel.getColor().getRed()*sourceImageWeight + destPixel.getColor().getRed()*destinationImageWeight)/2
        g = (sourcePixel.getColor().getGreen()*sourceImageWeight + destPixel.getColor().getGreen()*destinationImageWeight)/2
        b = (sourcePixel.getColor().getBlue()*sourceImageWeight + destPixel.getColor().getBlue()*destinationImageWeight)/2
  
        #set the new image's pixel to the cross-dissolved, warped pixel color
        setColor(newPixel, makeColor(r, g, b))

    #refresh the display to see morph
    repaint(newImage)

  #return the result
  return newImage


################################################################################
# Function name: interpolateLines
# Parameters:
#    -sourceLines: the list of source lines
#    -destinationLines: the list of destination lines
#    -numLines: the total number of feature lines
#    -currFrame: the current frame being morphed
#    -totalFrames: the total number of frames created
# Return:
#    An intermediate set of lines
# Description:
#    Uses the start and end points of the source and destination lines,
#    as well as the frame information to create an intermediate set of 
#    lines that is used to move the source lines to the destination lines
#    over a series of frames
################################################################################
def interpolateLines(sourceLines, destinationLines, numLines, currFrame, totalFrames):
  #initialize newLines to number of line segments needed
  newLines = [[[0, 0], [0, 0]]]
  for j in range(1, numLines):
    newLines += [[0, 0]]

  for i in range(0, numLines):
    PdeltaX = (destinationLines[i][0][0]-sourceLines[i][0][0])/(totalFrames+1)
    PdeltaY = (destinationLines[i][0][1]-sourceLines[i][0][1])/(totalFrames+1)
    QdeltaX = (destinationLines[i][1][0]-sourceLines[i][1][0])/(totalFrames+1)
    QdeltaY = (destinationLines[i][1][1]-sourceLines[i][1][1])/(totalFrames+1)
    newLines[i][0] = [ sourceLines[i][0][0] + (PdeltaX*(currFrame+1)), sourceLines[i][0][1] + (PdeltaY*(currFrame+1)) ]
    newLines[i][1] = [ sourceLines[i][1][0] + (QdeltaX*(currFrame+1)), sourceLines[i][1][1] + (QdeltaY*(currFrame+1)) ]
  return newLines


################################################################################
# Function name: calcXPrime
# Parameters:
#    -Pprime: the source line start point
#    -Qprime: the source line end point
#    -u: the distance along the feature line
#    -v: the perpendicular distance from the source pixel to the feature line
# Return:
#    The distance along the feature line
# Description:
#    Uses the Beier-Neely equation to calculate the new pixel value due 
#    to warping using various vector operations.  If the new value is 
#    out of the range of the image dimensions, the current value is used
################################################################################
def calcXPrime(Pprime, Qprime, u, v):
  QprimePprime = vectorSubtraction(Qprime, Pprime)
  QprimePprimePerp = perpendicular(QprimePprime)
  top = scalarMult(QprimePprimePerp, v)
  bottom = magnitude(QprimePprime)
  last = scalarDivision(top, bottom)
  mid = scalarMult(QprimePprime, u)
  value = [(Pprime[0] + mid[0] + last[0]), (Pprime[1] + mid[1] + last[1])]
  return value

################################################################################
# Function name: calcU
# Parameters:
#    -X: the current destination image pixel
#    -P: the destination line start point
#    -Q: the destination line end point
# Return:
#    The distance along the feature line
# Description:
#    Uses the Beier-Neely equation to calculate the distance along the 
#    feature line using various vector operations.  The value is between
#    0 and 1 if it lies along the line, otherwise u is outside the range
################################################################################
def calcU(X, P, Q):
  XP = vectorSubtraction(X, P)
  PQ = vectorSubtraction(Q, P)
  top = dotProduct(XP, PQ)
  bottom = magnitude(PQ)*magnitude(PQ)
  value = top/bottom
  return value  

################################################################################
# Function name: calcV
# Parameters:
#    -X: the current destination image pixel
#    -P: the destination line start point
#    -Q: the destination line end point
# Return:
#    The perpendicular distance from the current pixel to the line
# Description:
#    Uses the Beier-Neely equation to calculate the perpendicular distance
#    of the current destination pixel to the current feature line using 
#    various vector operations
################################################################################
def calcV(X, P, Q):
  XP = vectorSubtraction(X, P)
  PQ = vectorSubtraction(Q, P)
  PQperp = perpendicular(PQ)
  top = dotProduct(XP, PQperp)
  bottom = magnitude(PQ)
  value = top/bottom
  return value


######################
#  VECTOR FUNCTIONS  #
######################

################################################################################
# Function name: vectorSubtraction
# Description:  Subtracts the x and y components of the two input vectors
################################################################################
def vectorSubtraction(vector1, vector2):
  value = [(vector1[0] - vector2[0]), (vector1[1] - vector2[1])]
  return value

################################################################################
# Function name: vectorAddition
# Description:  Adds the x and y components of the two input vectors
################################################################################
def vectorAddition(vector1, vector2):
  value = [(vector1[0] + vector2[0]), (vector1[1] + vector2[1])]
  return value

################################################################################
# Function name: scalarAddition
# Description:  Adds the scalar to the x and y components of the input vector
################################################################################
def scalarAddition(vector, scalar):
  value = [ vector[0] + scalar, vector[1] + scalar ]
  return value

################################################################################
# Function name: scalarDivision
# Description:  Divides the x and y components of the input vector by the scalar
################################################################################
def scalarDivision(vector, scalar):
  value = [ vector[0]/scalar, vector[1]/scalar ]
  return value

################################################################################
# Function name: dotProduct
# Description:  Calculates the dot product of two input vectors
################################################################################
def dotProduct(vector1, vector2):
  value = (vector1[0] * vector2[0]) + (vector1[1] * vector2[1])
  return value

################################################################################
# Function name: magnitude
# Description:  Calculates the vector magnitude
################################################################################
def magnitude(vector):
  value = sqrt(vector[0]*vector[0] + vector[1]*vector[1])
  return value

################################################################################
# Function name: perpendicular
# Description:  Calculates the vector perpendicular to the input vector
################################################################################
def perpendicular(vector):
  value = [-vector[1], vector[0]]
  return value

################################################################################
# Function name: scalarMult
# Description:  Multiplies each the x component and the y component of the vector
#    by a scalar value
################################################################################
def scalarMult(vector, scalar):
  value = [vector[0]*scalar, vector[1]*scalar]
  return value
