#sets up the image and feature lines
def main():
  image = makePicture(pickAFile())
  sourceLines = [[[73, 50], [189, 49]]]
  destinationLines = [[[73, 129], [73, 50]]]
  warpMain(image, sourceLines, destinationLines, 1)

########################
#  MORPHING FUNCTIONS  #
########################

################################################################################
# Function name: warpMain
# Parameters:
#     -image: the original image before warping
#     -sourceLines: the feature lines in the source image
#     -destinationLines: the feature lines in the destination image
#     -numLines: the total number of feature lines
#     -filename: (optional) name to write pictures to
# Description:
#    A main function that sets up everything needed to perform the warp
#    including number of frames, output folder, calculation of line 
#    interpolation, the call to the warp function, and writing the new
#    images out to files.  Also prints status information for each frame 
#    including frame number and feature lines.
################################################################################
def warpMain(image, sourceLines, destinationLines, numLines, filename="warp"):
  #number of morphing frames between the images
  defaultFrameNumber = 1

  #set folder for output images
  setMediaFolder()

  #write out first and last frames
  writePictureTo(image, str(filename) + "0.jpg") #first frame is the source image

  #loop through number of frames and call morphing function
  for i in range(0, defaultFrameNumber-1):
    #calculate interpolated lines
    interpLines = interpolateLines(sourceLines, destinationLines, numLines, i, defaultFrameNumber)

    #print updated processing status
    printNow("Warping Frame: " + str(i+1))
    printNow("Source Image Feature Lines: " + str(sourceLines))
    printNow("Destination Image Feature Lines: " + str(interpLines))

    #call the morph function and save result into new image
    warpImage = imageWarp(image, sourceLines, interpLines, numLines)
    #write the current frame to file
    writePictureTo(warpImage, str(filename) + str(i+1) + ".jpg")

  #call once more for last frame
  printNow("Warping Frame: " + str(defaultFrameNumber))
  printNow("Source Image Feature Lines: " + str(sourceLines))
  printNow("Destination Image Feature Lines: " + str(destinationLines))
  warpImage = imageWarp(image, sourceLines, destinationLines, numLines)
  writePictureTo(warpImage, str(filename) + str(defaultFrameNumber) + ".jpg")

################################################################################
# Function name: imageWarp
# Parameters:
#     -image: the original image before warping
#     -interpLines: the intermediate feature lines
#     -destinationLines: the feature lines in the destination image
#     -numLines: the total number of feature lines
# Return:
#    A new image that is warped and cross dissolved according to the 
#    attributes for the current frame
# Description:
#    A morphing function based on the algorithm created by Beier and Neely 
#    as described in the 1992 SIGGRAPH paper "Feature-Based Image 
#    Metamorphosis" (http://www.hammerhead.com/thad/morph.html).
#    The technique uses feature lines in the source image and destination 
#    image to warp the two images towards the lines while cross-dissolving 
#    the images to blend them over time.
################################################################################
def imageWarp(image, sourceLines, interpLines, numLines):
  #CONSTANTS
  a = 0.5 #smoothness of warping
  b = 1.25 #relative line strength
  p = 0.25 #line size strength

  #get width and height of each image
  width = image.getWidth()
  height = image.getHeight()

  #create new blank image
  newImage = makeEmptyPicture(height, width)
  show(newImage)

  #loop through the pixels of both images
  for x in range(0, width):
    for y in range(0, height):

      #get pixel values
      destP = getPixel(image, x, y) #get current pixel from destination image
      newP = getPixel(newImage, x, y) #get current pixel from new image
      X = [x, y] #get x and y for the current pixel

      #initialize weight and displacement values
      DSUM = [0, 0] #intialize source displacement sum to 0
      weightsum = 0 #intialize source weight sum to 0

      #loop through list of lines and process each feature line in the images
      for i in range(0, numLines):

        #set the vector for the current line we are warping towards
        P = interpLines[i][0] #destination line start point
        Q = interpLines[i][1] #destination line end point
        PQ = vectorSubtraction(Q, P) #define destination line segment

        #calculate the u and v to warp the destination image
        u = calcU(X, P, Q) #u is the value along the feature line
        v = calcV(X, P, Q) #v is the perpendicular distance of the current pixel to the line

        #set the vector for the current line in the source image we are warping from
        Pprime = sourceLines[i][0] #source line start point
        Qprime = sourceLines[i][1] #source line start point

        #calculate the new pixel x and y values for the current source image pixel due to warping
        Xprime = calcXPrime(Pprime, Qprime, u, v)

        #find how far new pixels are from current pixels in the source and destination images
        displacement = vectorSubtraction(Xprime, X) #source image displacement

        #check to see if the effect of u
        #if u is greater than 1, distance is the length of the line from the pixel to the end point
        if(u >= 1):
          QX = vectorSubtraction(Q, X)
          dist = magnitude(QX)
        #if u is less than 0, distance is the length of the line from the pixel to the start point
        elif(u <= 0):
          PX = vectorSubtraction(P, X)
          dist = magnitude(PX)
        #otherwise distance is absolute value of v
        else:
          dist = abs(v)

        #find out contribution of current line to pixel warping
        #determine new xy's contribution based on its distance from the line
        length = magnitude(PQ) #length of current line
        weight = pow(length, p)/(a + dist)
        weight = pow(weight, b) #weight of current line

        #add the displacement sum for source and destination
        DSUM = vectorAddition(DSUM, scalarMult(displacement, weight))

        #add weight to weightsum
        weightsum = weightsum + weight

        #adjust displacement sum by weightsum
        DSUMweightsum = scalarDivision(DSUM, weightsum)

        #adjust new source and destination pixels by displacement sums
        Xprime = vectorAddition(X, DSUMweightsum)

        #get x and y values for new source pixel
        newSourceX = int(Xprime[0])
        newSourceY = int(Xprime[1])

        #if new source xy in range of source image, get that pixel
        if(newSourceX in range(0, width) and newSourceY in range(0, height)):
          sourcePixel = getPixel(image, newSourceX, newSourceY)
        else:
          #otherwise get current source image pixel
          sourcePixel = getPixel(image, x, y)
  
        #set the new image's pixel to the warped pixel color
        setColor(newP, sourcePixel.getColor())

    #refresh the display to see morph
    repaint(newImage)

  #return the result
  return newImage


################################################################################
# Function name: interpolateLines
# Parameters:
#    -sourceLines: the list of source lines
#    -destinationLines: the list of destination lines
#    -numLines: the total number of feature lines
#    -currFrame: the current frame being morphed
#    -totalFrames: the total number of frames created
# Return:
#    An intermediate set of lines
# Description:
#    Uses the start and end points of the source and destination lines,
#    as well as the frame information to create an intermediate set of 
#    lines that is used to move the source lines to the destination lines
#    over a series of frames
################################################################################
def interpolateLines(sourceLines, destinationLines, numLines, currFrame, totalFrames):
  #initialize newLines to number of line segments needed
  newLines = [[[0, 0], [0, 0]]]
  for j in range(1, numLines):
    newLines += [[0, 0]]

  for i in range(0, numLines):
    PdeltaX = (destinationLines[i][0][0]-sourceLines[i][0][0])/(totalFrames+1)
    PdeltaY = (destinationLines[i][0][1]-sourceLines[i][0][1])/(totalFrames+1)
    QdeltaX = (destinationLines[i][1][0]-sourceLines[i][1][0])/(totalFrames+1)
    QdeltaY = (destinationLines[i][1][1]-sourceLines[i][1][1])/(totalFrames+1)
    newLines[i][0] = [ sourceLines[i][0][0] + (PdeltaX*(currFrame+1)), sourceLines[i][0][1] + (PdeltaY*(currFrame+1)) ]
    newLines[i][1] = [ sourceLines[i][1][0] + (QdeltaX*(currFrame+1)), sourceLines[i][1][1] + (QdeltaY*(currFrame+1)) ]
  return newLines


################################################################################
# Function name: calcXPrime
# Parameters:
#    -Pprime: the source line start point
#    -Qprime: the source line end point
#    -u: the distance along the feature line
#    -v: the perpendicular distance from the source pixel to the feature line
# Return:
#    The distance along the feature line
# Description:
#    Uses the Beier-Neely equation to calculate the new pixel value due 
#    to warping using various vector operations.  If the new value is 
#    out of the range of the image dimensions, the current value is used
################################################################################
def calcXPrime(Pprime, Qprime, u, v):
  QprimePprime = vectorSubtraction(Qprime, Pprime)
  QprimePprimePerp = perpendicular(QprimePprime)
  top = scalarMult(QprimePprimePerp, v)
  bottom = magnitude(QprimePprime)
  last = scalarDivision(top, bottom)
  mid = scalarMult(QprimePprime, u)
  value = [(Pprime[0] + mid[0] + last[0]), (Pprime[1] + mid[1] + last[1])]
  return value

################################################################################
# Function name: calcU
# Parameters:
#    -X: the current destination image pixel
#    -P: the destination line start point
#    -Q: the destination line end point
# Return:
#    The distance along the feature line
# Description:
#    Uses the Beier-Neely equation to calculate the distance along the 
#    feature line using various vector operations.  The value is between
#    0 and 1 if it lies along the line, otherwise u is outside the range
################################################################################
def calcU(X, P, Q):
  XP = vectorSubtraction(X, P)
  PQ = vectorSubtraction(Q, P)
  top = dotProduct(XP, PQ)
  bottom = magnitude(PQ)*magnitude(PQ)
  value = top/bottom
  return value  

################################################################################
# Function name: calcV
# Parameters:
#    -X: the current destination image pixel
#    -P: the destination line start point
#    -Q: the destination line end point
# Return:
#    The perpendicular distance from the current pixel to the line
# Description:
#    Uses the Beier-Neely equation to calculate the perpendicular distance
#    of the current destination pixel to the current feature line using 
#    various vector operations
################################################################################
def calcV(X, P, Q):
  XP = vectorSubtraction(X, P)
  PQ = vectorSubtraction(Q, P)
  PQperp = perpendicular(PQ)
  top = dotProduct(XP, PQperp)
  bottom = magnitude(PQ)
  value = top/bottom
  return value


######################
#  VECTOR FUNCTIONS  #
######################

################################################################################
# Function name: vectorSubtraction
# Description:  Subtracts the x and y components of the two input vectors
################################################################################
def vectorSubtraction(vector1, vector2):
  value = [(vector1[0] - vector2[0]), (vector1[1] - vector2[1])]
  return value

################################################################################
# Function name: vectorAddition
# Description:  Adds the x and y components of the two input vectors
################################################################################
def vectorAddition(vector1, vector2):
  value = [(vector1[0] + vector2[0]), (vector1[1] + vector2[1])]
  return value

################################################################################
# Function name: scalarAddition
# Description:  Adds the scalar to the x and y components of the input vector
################################################################################
def scalarAddition(vector, scalar):
  value = [ vector[0] + scalar, vector[1] + scalar ]
  return value

################################################################################
# Function name: scalarDivision
# Description:  Divides the x and y components of the input vector by the scalar
################################################################################
def scalarDivision(vector, scalar):
  value = [ vector[0]/scalar, vector[1]/scalar ]
  return value

################################################################################
# Function name: dotProduct
# Description:  Calculates the dot product of two input vectors
################################################################################
def dotProduct(vector1, vector2):
  value = (vector1[0] * vector2[0]) + (vector1[1] * vector2[1])
  return value

################################################################################
# Function name: magnitude
# Description:  Calculates the vector magnitude
################################################################################
def magnitude(vector):
  value = sqrt(vector[0]*vector[0] + vector[1]*vector[1])
  return value

################################################################################
# Function name: perpendicular
# Description:  Calculates the vector perpendicular to the input vector
################################################################################
def perpendicular(vector):
  value = [-vector[1], vector[0]]
  return value

################################################################################
# Function name: scalarMult
# Description:  Multiplies each the x component and the y component of the vector
#    by a scalar value
################################################################################
def scalarMult(vector, scalar):
  value = [vector[0]*scalar, vector[1]*scalar]
  return value