######################################
#  SLIDE SHOW AND MOVIE TRANSITIONS  #
######################################

#################################################################################
# Function name: crossDissolve
# Parameters:
#     -firstPicture: the original image before cross-dissolve
#     -secondPicture: the final image we are cross-dissolving to
#     -filename: (optional) filename for writing output
# Description:
#    Takes in two images of the same size and outputs a series of frames
#    with the default number of frames generated being 5.  The transition 
#    effect is of the first image blending into the second image.  The frames 
#    are written to a directory with the frame number as the last character in 
#    the filename.
#################################################################################
def crossDissolve(firstPicture, secondPicture, filename="crossDissolve"):
  #default  number of tweening frames
  frameNum = 5

  #set the output folder and write first and last frames
  #the first frame is firstPicture, the last is secondPicture
  setMediaFolder()
  writePictureTo(firstPicture, str(filename) + "0.jpg")
  writePictureTo(secondPicture, str(filename) + str(frameNum+1) + ".jpg")

  #percentage the image is faded by
  fadeIncrement = 1.0/(frameNum+1)

  #get height and width of input images
  width = getWidth(firstPicture)
  height = getHeight(firstPicture)

  #create new blank picture to hold output
  newImage = makeEmptyPicture(height, width)

  #loop through each frame to be created
  for i in range(1, frameNum+1):
    printNow("Cross-dissolving Frame " + str(i))

    for y in range(0, height):
      for x in range(0, width):
        #get pixels in first and second picture
        p1 = getPixel(firstPicture, x, y)
        p2 = getPixel(secondPicture, x, y)

        #get pixel for new image
        newPixel = getPixel(newImage, x, y)

        #get RGB values from first image
        firstRed = getRed(p1)
        firstGreen = getGreen(p1)
        firstBlue = getBlue(p1)

        #get RGB values from second image    
        secondRed = getRed(p2)
        secondGreen = getGreen(p2)
        secondBlue = getBlue(p2)

        #calculate current fade intensity
        intensity = 2 * i * fadeIncrement

        #adjust colors by intensity
        newRed = (((2.0-intensity)*firstRed) + (intensity*secondRed))/2
        newGreen = (((2.0-intensity)*firstGreen) + (intensity*secondGreen))/2
        newBlue = (((2.0-intensity)*firstBlue) + (intensity*secondBlue))/2

        #set new RGB value for blend
        setColor(newPixel, makeColor(newRed, newGreen, newBlue))

      #write new frame to file
      writePictureTo(newImage, str(filename) + str(i) + ".jpg")


#################################################################################
# Function name: fadeToBlack
# Parameters:
#     -image: the picture to be faded
#     -filename: (optional) filename for writing output
# Description:
#    Takes in an image, and fades the image to black over a series of frames
#    with the default number of frames generated being 5.  The frames are 
#    written to a directory with the frame number as the last character in the
#    filename.
#################################################################################
def fadeToBlack(image, filename="fadeToBlack"):
  #default  number of tweening frames
  frameNum = 5

  #set the output folder and write first frame
  #the first frame is image
  setMediaFolder()
  writePictureTo(image, str(filename) + "0.jpg")

  #percentage the image is faded by
  fadeIncrement = 1.0/(frameNum+1)

  #get height and width of input images
  width = getWidth(image)
  height = getHeight(image)

  #create new blank picture to hold output
  newImage = makeEmptyPicture(height, width)

  #loop through each frame to be created
  for i in range(1, frameNum+1):
    printNow("Fading frame " + str(i))

    for y in range(0, height):
      for x in range(0, width):
        p1 = getPixel(image, x, y) #pixel from image
        p2 = getPixel(newImage, x, y) #pixel from new image

        #get RGB values from image
        r = getRed(p1)
        g = getGreen(p1)
        b = getBlue(p1)

        #calculate the current intensity value
        intensity = 1 - (i * fadeIncrement)

        #set the new RGB values for the image
        setColor(p2, makeColor((r * intensity), (g * intensity), (b * intensity)))

      #write new frame to file
      writePictureTo(newImage, str(filename) + str(i) + ".jpg")


#################################################################################
# Function name: rollRight
# Parameters:
#     -image1: starting image
#     -image2: image we are transitioning into
#     -filename: (optional) filename for writing output
# Description:
#    Takes in two images of the same size and outputs a series of frames
#    with the default number of frames generated being 5.  The transition 
#    effect is of the second image being unrolled over the first image.  
#    The frames are written to a directory with the frame number as the 
#    last character in the filename.
#################################################################################
def rollRight(image1, image2, filename="rollRight"):
  #define constant for number of tweening frames generated
  frameNum = 5

  #set the output folder and write first and last frames
  #the first frame is image2, the last is image2
  setMediaFolder()
  writePictureTo(image1, str(filename) + "0.jpg")
  writePictureTo(image2, str(filename) + str(frameNum+1) + ".jpg")

  #percentage the image is rolled by
  rollIncrement = 1.0/(frameNum+1)

  #get height and width of input images
  width = getWidth(image1)
  height = getHeight(image1)

  #create a new blank picture to hold output
  newImage = makeEmptyPicture(height, width)

  #loop until all frames processed
  for i in range(1, frameNum+1):
    printNow("Rolling frame " + str(i))

    for y in range(0, height):
      for x in range(0, width):
        p1 = getPixel(image1, x, y)
        p2 = getPixel(image2, x, y)

        #calculate percentage of roll
        percentage = i * rollIncrement

        #replace image1 values with image2 if less than percentage width
        if(x <= (width * percentage)):
          newPixel = getPixel(newImage, x, y)
          setColor(newPixel, getColor(p2))
        #else new image gets the current pixel of image1
        else:
          newPixel = getPixel(newImage, x, y)
          setColor(newPixel, getColor(p1))

    #write frame out to file
    writePictureTo(newImage, str(filename) + str(i) + ".jpg")


#################################################################################
# Function name: slideRight
# Parameters:
#     -image1: starting image
#     -image2: image we are transitioning into
#     -filename: (optional) filename for writing output
# Description:
#    Takes in two images of the same size and outputs a series of frames
#    with the default number of frames generated being 5.  The transition 
#    effect is of the second image sliding over the first image.  The frames 
#    are written to a directory with the frame number as the last character 
#    in the filename.
#################################################################################
def slideRight(image1, image2, filename="slideRight"):
  #define constant for number of tweening frames generated
  frameNum = 5

  #set the output folder and write first and last frames
  #the first frame is image2, the last is image2
  setMediaFolder()
  writePictureTo(image1, str(filename) + "0.jpg")
  writePictureTo(image2, str(filename) + str(frameNum+1) + ".jpg")

  #percentage the image is rolled by
  slideIncrement = 1.0/(frameNum+1)

  #get height and width of input images
  width = getWidth(image1)
  height = getHeight(image1)

  #create a new blank picture to hold output
  newImage = makeEmptyPicture(height, width)

  #loop until all frames processed
  for i in range(1, frameNum+1):
    printNow("Sliding frame " + str(i))

    for y in range(0, height):
      for x in range(0, width):
        #calculate percentage of roll
        percentage = i * slideIncrement

        #replace image1 values with image2 if less than percentage width
        if(x < (width * percentage)):
          p2 = getPixel(image2, int(width + x  - (width * percentage)), y)
          newPixel = getPixel(newImage,  x, y)
          setColor(newPixel, getColor(p2))
        #else new image gets the current pixel of image1
        else:
          p1 = getPixel(image1, x, y)
          newPixel = getPixel(newImage, x, y)
          setColor(newPixel, getColor(p1))

    #write frame out to file
    writePictureTo(newImage, str(filename) + str(i) + ".jpg")